#!/bin/sh

#################
# SHELL OPTIONS #
#################

# make wildcard globbing with no match return null instead of literal
shopt -s nullglob
# make wildcard globbing case-insensitive (such as WAV vs. wav)
shopt -s nocaseglob

########################
# FUNCTION DEFINITIONS #
########################

# function to wait for user interaction
pause()
{
	echo "Press any key to continue . . ."
	while [ true ] ; do
		# poll keyboard every second to read one character
		# suppresses echo of character to console
		read -s -t 1 -n 1
		if [ $? = 0 ] ; then
			# quit loop when user actually makes an entry
			break;
		fi
	done
}

# function to run any time script needs to abort
enditall()
{
	pause
	# close terminal without user intervention
	osascript -e 'tell application "Terminal" to close (every window whose name contains "EFE2IMG")' & exit 1
}

###############
# MAIN SCRIPT #
###############

# change terminal window name
echo -n -e "\033]0;EFE2IMG\007"

# change terminal window dimensions
echo -n -e "\033[8;40;80t"

# get directory that launched script and use it as current directory
DIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
cd "${DIR}"

# create the [EFE_IN] directory (with error suppression if already exists)
mkdir ./\[EFE_IN\] 2>/dev/null

clear
echo "###############################################################"
echo "# Ensoniq EFA/EFE to IMG and HFE (ASR series) Converter v0.70 # (ASR 1600K) "
echo "###############################################################"
echo ""
echo "Input files must be Ensoniq instruments (EFE/EFA/INS) placed within the "
echo "[EFE_IN] folder. Files extracted by various Ensoniq utilities such as "
echo "EpsLin, EPSDisk, EnsoniqFS, and Translator are accepted. "
echo ""
echo "Sub-folders are supported and will be preserved in the output structure. "
echo ""
echo "New IMG files with corresponding names appear in the [IMG_ASR_OUT] folder when "
echo "the encoding task is finished. In order to maximize storage, there will be no "
echo "operating system, so the images will not be bootable. "
echo ""
echo "Additionally, all created IMGs will automatically be converted into ASR "
echo "compatible HFEs which will be found in the [HFE_ASR_OUT] folder. "
echo ""
pause
echo ""

# check for required tools and metadata
if [ ! -f "./tools/epslin" ]; then
	echo "Critical Error: epslin cannot be found. Aborting! \n"
	enditall
fi

if [ ! -f "./tools/App/hxcfe" ]; then
	echo "Critical Error: hxcfe cannot be found. Aborting! \n"
	enditall
fi

if [ ! -f "./tools/Frameworks/libhxcfe.dylib" ]; then
	echo "Critical Error: libhxcfe cannot be found. Aborting! \n"
	enditall
fi

if [ ! -f "./tools/Frameworks/libusbhxcfe.dylib" ]; then
	echo "Critical Error: libusbhxcfe cannot be found. Aborting! \n"
	enditall
fi

# check for number of EFA/efa/EFE/efe/INS/ins files in the [EFE_IN] folder
# note: done recursively so sub-directories are supported
foundEFEs=$(find ./\[EFE_IN\] -type f \( -iname '*.efe' -o -iname '*.ins' -o -iname '*.efa' \) | wc -l)
if [ $foundEFEs -eq 0 ]; then
	echo "Critical Error: No EFA, EFE, or INS files found to convert. Aborting! \n"
	enditall
fi

# duplicate all EFA/EFE/INS files so that processing will not change originals!
# retains sub-directory structure
echo "Preparing conversion. Please wait... "
cp -r "./[EFE_IN]" ./temp

# create a variable to represent maximum file size which can fit on one disk image
maxsize=1626624

# EFEs which are too large for standard ASR 1600K disks will be auto-split into multiple 1600K ready volumes
find ./temp -type f \( -iname '*.efe' -o -iname '*.ins' -o -iname '*.efa' \) -print0 |
while IFS= read -r -d '' ensfile; do
	filesize=$(stat -f %z "${ensfile}")
	# too big? then split
	if (( filesize > maxsize )); then
		./tools/epslin -s "${ensfile}" eps > /dev/null
		rm "${ensfile}" > /dev/null
	fi
done

echo "Storing all EFAs and EFEs to new IMGs... "
echo ""

# process all EFA/EFE/INS files
find ./temp -type f \( -iname '*.efe' -o -iname '*.ins' -o -iname '*.efa' \) -print0 |
while IFS= read -r -d '' ensfile; do
	# get directory of input EFE
	enspath=$(dirname "${ensfile}")
	# strip path to derive filename only (without path or extension)
	ensname=$(basename -- "${ensfile}")
	ensname=${ensname%.*}
	# create new blank disk image in ASR 1600K format using filename of EFE
	./tools/epslin -q -l DISK000 -fi "${enspath}/${ensname}.img" asr > /dev/null
	# store EFE to newly created disk image
	./tools/epslin -p1 "${enspath}/${ensname}.img" "${ensfile}" > /dev/null
	# give warning if EFE could not be stored to IMG
	if [ "$?" = 0 ]; then
		# success
		echo "${ensname} was added to the disk image. "
	else
		# failure
		echo "Warning: ${ensname} could not be written to the disk image. "
	fi # end of errorlevel routine
done

# process all IMG files
echo "\nGenerating ASR-type HFEs. This may take some time, please wait... "
find ./temp -type f \( -iname '*.img' \) -print0 |
while IFS= read -r -d '' imgfile; do
	# get directory of input IMG
	imgpath=$(dirname "${imgfile}")
	# strip path to derive filename only (without path or extension)
	imgname=$(basename -- "{$imgfile}")
	imgname=${imgname%.*}
	# convert any found IMG to ASR 1600K type HFE
	./tools/App/hxcfe -finput:"${imgfile}" -conv -uselayout:ENSONIQ_HD_1600KB -ifmode:IBMPC_HD_FLOPPYMODE -foutput:"${imgpath}/${imgname}.hfe" > /dev/null
done

echo "Finalizing disk images. Please wait... "

# should ASR IMG output folder already exist then merge IMGs
# from temp into it, otherwise IMGs copy to new output folder
rsync -a ./temp/ "./[IMG_ASR_OUT]/"

# remove all non-IMG files from IMG output folder
find "./[IMG_ASR_OUT]" -type f ! -iname '*.img' -delete

# remove all non-HFE files since conversion is now complete
find ./temp -type f ! -iname '*.hfe' -delete

# should ASR HFE output folder already exist then merge temp into it
# otherwise, simply rename to save time and disk usage
if [ -d "./[HFE_ASR_OUT]" ]; then
	rsync -a ./temp/ "./[HFE_ASR_OUT]/"
	rm -r ./temp
else
	# then rename temporary directory structure to ASR HFE output
	mv ./temp "./[HFE_ASR_OUT]"
fi

echo ""
echo "Process complete. "
echo ""
echo "IMG and HFE files should now be present within the [IMG_ASR_OUT] and "
echo "[HFE_ASR_OUT] folders. "
echo ""
echo "Note: The resulting IMG and HFE files will only work with Ensoniq models with "
echo "ASR format compatibility and HD floppy drives; no EPS models, sorry. "
echo ""
pause

# close terminal without user intervention
osascript -e 'tell application "Terminal" to close (every window whose name contains "EFE2IMG")' & exit 0
