#!/bin/sh

#################
# SHELL OPTIONS #
#################

# make wildcard globbing with no match return null instead of literal
shopt -s nullglob
# make wildcard globbing case-insensitive (such as WAV vs. wav)
shopt -s nocaseglob

########################
# FUNCTION DEFINITIONS #
########################

# function to wait for user interaction
pause()
{
	echo "Press any key to continue . . ."
	while [ true ] ; do
		# poll keyboard every second to read one character
		# suppresses echo of character to console
		read -s -t 1 -n 1
		if [ $? = 0 ] ; then
			# quit loop when user actually makes an entry
			break;
		fi
	done
}

# function to run any time script needs to abort
enditall()
{
	pause
	# close terminal without user intervention
	osascript -e 'tell application "Terminal" to close (every window whose name contains "EFE2IMG")' & exit 1
}

###############
# MAIN SCRIPT #
###############

# change terminal window name
echo -n -e "\033]0;EFE2IMG\007"

# change terminal window dimensions
echo -n -e "\033[8;40;80t"

# get directory that launched script and use it as current directory
DIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
cd "${DIR}"

# create the [EFE_IN] directory (with error suppression if already exists)
mkdir ./\[EFE_IN\] 2>/dev/null

clear
echo "##############################################"
echo "# Ensoniq EFE to IMG and HFE Converter v0.70 # (EPS16 2550K) "
echo "##############################################"
echo ""
echo "Input files must be Ensoniq instruments (EFE/INS) placed within the [EFE_IN] "
echo "folder. Files extracted by various Ensoniq utilities such as EpsLin, EPSDisk, "
echo "EnsoniqFS, and Translator are accepted. "
echo ""
echo "Sub-folders are supported and will be preserved in the output structure. "
echo ""
echo "New HFE files with corresponding names appear in the [HFE_E16_OUT] folder when "
echo "the encoding task is finished. In order to maximize storage, there will be no "
echo "operating system, so the images will not be bootable. "
echo ""
echo "The created HFEs will only work in EPS16 systems with a floppy emulator, as "
echo "both the original EPS and standard floppy drives are not capable of using "
echo "SuperDisk images. "
echo ""
pause
echo ""

# check for required tools and metadata
if [ ! -f "./tools/epslin" ]; then
	echo "Critical Error: epslin cannot be found. Aborting! \n"
	enditall
fi

if [ ! -f "./tools/App/hxcfe" ]; then
	echo "Critical Error: hxcfe cannot be found. Aborting! \n"
	enditall
fi

if [ ! -f "./tools/Frameworks/libhxcfe.dylib" ]; then
	echo "Critical Error: libhxcfe cannot be found. Aborting! \n"
	enditall
fi

if [ ! -f "./tools/Frameworks/libusbhxcfe.dylib" ]; then
	echo "Critical Error: libusbhxcfe cannot be found. Aborting! \n"
	enditall
fi

if [ ! -f "./tools/EPS16_DD_2550KB.xml" ]; then
	echo "Critical Error: EPS16 SuperDisk XML cannot be found. Aborting! \n"
	enditall
fi

# check for number of EFE/efe/INS/ins files in the [EFE_IN] folder
# note: done recursively so sub-directories are supported
foundEFEs=$(find ./\[EFE_IN\] -type f \( -iname '*.efe' -o -iname '*.ins' \) | wc -l)
if [ $foundEFEs -eq 0 ]; then
	echo "Critical Error: No EFE or INS files found to convert. Aborting! \n"
	enditall
fi

# duplicate all EFE/INS files so that processing will not change originals!
# retains sub-directory structure
echo "Preparing conversion. Please wait... "
cp -r "./[EFE_IN]" ./temp

# create a variable to represent maximum file size which can fit on one disk image
maxsize=2593280

# EFEs which are too large for EPS16 2550K disks cannot fit in EPS16 RAM anyway, so...
find ./temp -type f \( -iname '*.efe' -o -iname '*.ins' \) -print0 |
while IFS= read -r -d '' ensfile; do
	filesize=$(stat -f %z "${ensfile}")
	# too big? then delete without pointless splitting
	if (( filesize > maxsize )); then
		rm "${ensfile}" > /dev/null
	fi
done

echo "Storing all EFEs to new IMGs... "
echo ""

# process all EFE/INS files
find ./temp -type f \( -iname '*.efe' -o -iname '*.ins' \) -print0 |
while IFS= read -r -d '' ensfile; do
	# get directory of input EFE
	enspath=$(dirname "${ensfile}")
	# strip path to derive filename only (without path or extension)
	ensname=$(basename -- "${ensfile}")
	ensname=${ensname%.*}
	# create new blank disk image in EPS 800K format using filename of EFE
	./tools/epslin -q -l DISK000 -fi "${enspath}/${ensname}.img" e16sd > /dev/null
	# store EFE to newly created disk image
	./tools/epslin -p1 "${enspath}/${ensname}.img" "${ensfile}" > /dev/null
	# give warning if EFE could not be stored to IMG
	if [ "$?" = 0 ]; then
		# success
		echo "${ensname} was added to the disk image. "
	else
		# failure
		echo "Warning: ${ensname} could not be written to the disk image. "
	fi # end of errorlevel routine
done

# process all IMG files
echo "\nGenerating EPS16-type HFEs. This may take some time, please wait... "
find ./temp -type f \( -iname '*.img' \) -print0 |
while IFS= read -r -d '' imgfile; do
	# get directory of input IMG
	imgpath=$(dirname "${imgfile}")
	# strip path to derive filename only (without path or extension)
	imgname=$(basename -- "{$imgfile}")
	imgname=${imgname%.*}
	# convert any found IMG to EPS 800K type HFE
	./tools/App/hxcfe -finput:"${imgfile}" -conv -uselayout:./tools/EPS16_DD_2550KB.XML -ifmode:GENERIC_SHUGART_DD_FLOPPYMODE -foutput:"${imgpath}/${imgname}.hfe" > /dev/null
done

echo "Finalizing disk images. Please wait... "

# remove all non-HFE files since conversion is now complete
find ./temp -type f ! -iname '*.hfe' -delete

# should EPS16 HFE output folder already exist then merge temp into it
# otherwise, simply rename to save time and disk usage
if [ -d "./[HFE_E16_OUT]" ]; then
	rsync -a ./temp/ "./[HFE_E16_OUT]/"
	rm -r ./temp
else
	# then rename temporary directory structure to EPS16 HFE output
	mv ./temp "./[HFE_E16_OUT]"
fi

echo ""
echo "Process complete. "
echo ""
echo "EPS16 SuperDisk compatible HFE files should now be present within the "
echo "[HFE_E16_OUT] folder. "
echo ""
pause

# close terminal without user intervention
osascript -e 'tell application "Terminal" to close (every window whose name contains "EFE2IMG")' & exit 0
